@@ -1,7 +1,7 @@
 void mgcv(double *yd,double *Xd,double *Cd,double *wd,double *Sd,
      double *pd, double *sp,int *offd,int *dimd,int *md,
      int *nd,int *qd,int *rd,double *sig2d, double *Vpd, double *edf, double *conv_tol, int *ms_max_half,
-	 double *ddiag,int *idiag,double *sdiag, int *direct_mesh,double *min_edf,double *gcvubre);
+	 double *ddiag,int *idiag,double *sdiag, int *direct_mesh,double *min_edf,double *gcvubre,double *target_edf);
 void RGAMsetup(double *Xd,double *Cd,double *Sd,double *UZd,double *Xud,int *xu,double *xpd,
              int *offd,double *xd,int  *md,int  *nd,int *dfd,int *nsdfd,int *dim,int *s_type, int *p_order,
              int *by_exists, double *byd,double *knots, int *n_knots);
